USE [igops]
GO

DROP VIEW IF EXISTS deleted_review_items_v
GO

CREATE VIEW deleted_review_items_v AS
  WITH deleted_review_items AS (
  SELECT
      item.item_type,
      CASE
          WHEN item.item_type = 'PERMISSION' THEN perm.name
          WHEN item.item_type = 'ACCOUNT' THEN acct.name
          WHEN item.item_type = 'ROLE' THEN role.name
          WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.name
      END AS review_item_name,
      item.id AS item_id,
      usr.display_name,
      usr.unique_user_id,
      instance.id AS review_instance_id,
      instance.status,
      defn.name AS review_name,
      CASE
          WHEN item.item_type = 'ROLE' THEN role.time_deletion
          WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.update_time
      END AS role_brole_time_deletion,
      CASE
          WHEN item.item_type = 'PERMISSION' THEN perm.base_application
          WHEN item.item_type = 'ACCOUNT' THEN acct.base_application
      END AS base_application,
      CASE
          WHEN item.item_type = 'PERMISSION' THEN perm.sversion
          WHEN item.item_type = 'ACCOUNT' THEN acct.sversion
      END AS sversion
  FROM review_item item
      JOIN suser usr ON item.unique_user_id = usr.unique_user_id AND usr.effective = 1
      JOIN review_instance instance ON item.review_instance_id = instance.id
      JOIN ( 
          SELECT review_instance.id AS inst_id,
              CASE
                  WHEN staged_defn.name IS NULL THEN defn_1.name
                  ELSE staged_defn.name
              END AS name
              FROM review_definition defn_1
                  JOIN review_instance ON defn_1.id = review_instance.review_definition
                  LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
      LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1 AND perm.deleted = 1
      LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1 AND acct.deleted = 1
      LEFT JOIN (
          SELECT
              role_policy.name,
              role_policy.id,
              (SELECT max(update_time) FROM role_policy rp WHERE rp.unique_policy_id = role_policy.unique_policy_id) AS time_deletion
          FROM role_policy WHERE deleted = 1 
              AND unique_policy_id NOT IN (SELECT unique_policy_id FROM role_policy WHERE deleted = 0)
      ) role ON role.id = item.role_id
      LEFT JOIN business_role brole ON brole.id = item.business_role_id AND brole.approval_state = 'ARCHIVE'
  )
  SELECT
      dri.item_type,
      dri.review_item_name,
      dri.item_id,
      dri.display_name,
      dri.unique_user_id,
      dri.review_instance_id,
      dri.status,
      dri.review_name,
      CASE
          WHEN dri.item_type IN ('BUSINESS_ROLE', 'ROLE') THEN dri.role_brole_time_deletion
          WHEN dri.item_type IN ('ACCOUNT', 'PERMISSION') THEN 
              (SELECT 
                  min(sversion.timestamp) 
              FROM application_resolution app 
                  JOIN sversion ON app.application_resolver = sversion.data_producer
              WHERE sversion.id > dri.sversion AND app.application_id = dri.base_application
                  AND sversion.data_production > (SELECT min(data_production) FROM sversion sv WHERE sv.data_producer = sversion.data_producer))
      END AS time_deletion
  FROM deleted_review_items dri
  WHERE dri.review_item_name IS NOT NULL AND dri.status IN ('PREVIEW', 'RUNNING');  
GO

GRANT SELECT ON deleted_review_items_v TO ig_report_role
GO